@@ -5,25 +5,27 @@ use strict;
 use warnings;
 use utf8;
 
-our $VERSION = 0.17;
+our $VERSION = 0.18;
 
 use Encode::Detect::CJK qw/detect/;
 use Encode;
 use HTTP::Tiny;
-use Parallel::ForkManager;
 use Term::ProgressBar;
 use IO::Uncompress::Gunzip qw(gunzip);
 use URI::Escape;
+use Parallel::Loops;
 
 our $DEFAULT_URL_CONTENT = '';
-our %DEFAULT_HEADER = (
-    'Accept' => 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8', 
+our %DEFAULT_HEADER      = (
+    'Accept' =>
+      'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
     'Accept-Charset'  => 'gb2312,utf-8;q=0.7,*;q=0.7',
     'Accept-Encoding' => "gzip",
-    'Accept-Language' => 'zh,zh-cn;q=0.8,en-us;q=0.5,en;q=0.3', 
+    'Accept-Language' => 'zh,zh-cn;q=0.8,en-us;q=0.5,en;q=0.3',
     'Connection'      => 'keep-alive',
-    'User-Agent' => 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:29.0) Gecko/20100101 Firefox/29.0', 
-    'DNT' => 1, 
+    'User-Agent' =>
+'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:29.0) Gecko/20100101 Firefox/29.0',
+    'DNT' => 1,
 );
 
 sub new {
@@ -43,32 +45,32 @@ sub _init_browser {
     my $http = HTTP::Tiny->new( default_headers => \%h, );
 
     return $http;
-} ## end sub init_browser
+}
 
 sub request_urls {
     my ( $self, $src_arr, %opt ) = @_;
+
     # arr : url / { url => .. }
     # process_sub => sub { my ($html_ref) = @_ ; ... }
-    my $arr = $opt{select_url_sub} ? $opt{select_url_sub}->( $src_arr ) : $src_arr;
 
-    my $cnt = 0;
-    my @res_arr =();
-    
+    my $arr =
+      $opt{select_url_sub} ? $opt{select_url_sub}->($src_arr) : $src_arr;
     my $iter_sub = sub {
         my ($r) = @_;
-        return $opt{data_sub}->(@_) if($opt{no_auto_request_url});
+        return $opt{data_sub}->(@_) if ( $opt{no_auto_request_url} );
 
         my ( $url, $post_data ) =
-        ref($r) eq 'HASH' ? @{$r}{qw/url post_data/} 
-        : ( $r, undef );
+          ref($r) eq 'HASH'
+          ? @{$r}{qw/url post_data/}
+          : ( $r, undef );
 
-        my @procss_data = ($url, $post_data); 
-        if($url=~/^https?:/){
+        my @procss_data = ( $url, $post_data );
+        if ( $url =~ /^https?:/ ) {
             my $h = $self->request_url( $url, $post_data );
             @procss_data = ( \$h );
         }
-        my @return_data = exists $opt{data_sub} ?
-        $opt{data_sub}->(@procss_data) : @procss_data;
+        my @return_data =
+          exists $opt{data_sub} ? $opt{data_sub}->(@procss_data) : @procss_data;
         return @return_data;
     };
 
@@ -76,27 +78,25 @@ sub request_urls {
     $progress = Term::ProgressBar->new( { count => scalar(@$arr) } )
       if ( $opt{verbose} );
 
+    my $pm  = Parallel::Loops->new( $self->{max_process_num} );
+    my $cnt = [0];
+    $pm->share($cnt);
 
-    my $pm = Parallel::ForkManager->new( $self->{max_process_num} );
-    $pm->run_on_finish(sub {
-            my ( $pid, $exit_code, $ident, $exit, $core, $data ) = @_;
+    my %res_hash = $pm->foreach(
+        [ 0 .. $#$arr ],
+        sub {
+            my $i   = $_;
+            my $r   = $arr->[$i];
+            my @res = $iter_sub->($r);
+            my $k   = $#res == 0 ? $res[0] : \@res;
+            $cnt->[0]++;
+            $progress->update( $cnt->[0] ) if ( $opt{verbose} );
+            return ( $i, $k );
+        }
+    );
 
-            $cnt++;
-            $progress->update($cnt) if ( $opt{verbose} );
-            return unless ($data);
-
-            my ( $id, @res ) = @$data;
-            $res_arr[$id] = $#res==0 ? $res[0] : \@res;
-        });
-
-    for my $i ( 0 .. $#$arr ) {
-        my $pid = $pm->start and next;
-        my $r = $arr->[$i];
-        my @res = $iter_sub->($r);
-        $pm->finish( 0, [ $i, @res ]);
-    }
-
-    $pm->wait_all_children;
+    my @res_arr = map { $res_hash{$_} } ( 0 .. $#$arr );
+    $progress->update( scalar(@$arr) ) if ( $opt{verbose} );
 
     return \@res_arr;
 }
@@ -104,7 +104,7 @@ sub request_urls {
 sub request_urls_iter {
     my ( $self, $url, %o ) = @_;
 
-    my $html = $self->request_url($url, $o{post_data});
+    my $html = $self->request_url( $url, $o{post_data} );
 
     my $info      = $o{info_sub}->( \$html )      || {};
     my $data_list = $o{data_list_sub}->( \$html ) || [];
@@ -112,26 +112,27 @@ sub request_urls_iter {
     return ( $info, $data_list ) if ( $o{stop_sub}->( $info, $data_list ) );
 
     my $url_list = $o{url_list_sub}->( \$html ) || [];
-    my $select_url_list = $o{select_url_sub}->( $url_list );
+    my $select_url_list = $o{select_url_sub}->($url_list);
     for my $u (@$select_url_list) {
         return ( $info, $data_list ) if ( $o{stop_sub}->( $info, $data_list ) );
 
         my ( $u_url, $u_post_data ) =
-        ref($u) eq 'HASH' ? @{$u}{qw/url post_data/} 
-        : ( $u, undef );
+          ref($u) eq 'HASH'
+          ? @{$u}{qw/url post_data/}
+          : ( $u, undef );
 
-        my $c = $self->request_url($u_url, $u_post_data);
+        my $c = $self->request_url( $u_url, $u_post_data );
 
         my $fs = $o{data_list_sub}->( \$c ) || [];
         push @$data_list, @$fs;
     }
 
     return ( $info, $data_list );
-} ## end sub get_tiezi_ref
+}
 
 sub request_url {
     my ( $self, $url, $form ) = @_;
-    return $DEFAULT_URL_CONTENT unless($url);
+    return $DEFAULT_URL_CONTENT unless ($url);
 
     my $c;
     for my $i ( 1 .. $self->{retry} ) {
@@ -144,14 +145,14 @@ sub request_url {
 }
 
 sub format_post_content {
-    my ($self, $form) = @_;
+    my ( $self, $form ) = @_;
 
     my @params;
-    while(my ($k, $v) = each %$form){
-        push @params, uri_escape($k)."=".uri_escape($v);
+    while ( my ( $k, $v ) = each %$form ) {
+        push @params, uri_escape($k) . "=" . uri_escape($v);
     }
 
-    my $post_str = join("&", @params);
+    my $post_str = join( "&", @params );
     return $post_str;
 }
 
@@ -159,21 +160,25 @@ sub request_url_simple {
     my ( $self, $url, $form ) = @_;
 
     my $res =
-        $form
-      ? 
-      $self->{browser}->request('POST', $url, {
-              content => $self->format_post_content($form),
-               headers => { 
-                   %DEFAULT_HEADER, 
-                   'content-type' => 'application/x-www-form-urlencoded' }, 
-          })
+      $form
+      ? $self->{browser}->request(
+        'POST', $url,
+        {
+            content => $self->format_post_content($form),
+            headers => {
+                %DEFAULT_HEADER,
+                'content-type' => 'application/x-www-form-urlencoded'
+            },
+        }
+      )
       : $self->{browser}->get($url);
     return $DEFAULT_URL_CONTENT unless ( $res->{success} );
 
     my $html;
     my $content = $res->{content};
-    if ( $res->{headers}{'content-encoding'} and
-         $res->{headers}{'content-encoding'} eq 'gzip' ) {
+    if (    $res->{headers}{'content-encoding'}
+        and $res->{headers}{'content-encoding'} eq 'gzip' )
+    {
         gunzip \$content => \$html, MultiStream => 1, Append => 1;
     }
 
